/*
 *
 *	CMesh.h
 *	Jack Hoxley
 *
 */


#ifndef INC_CMESH_H
#define INC_CMESH_H

#include <d3dx9.h>

class CMesh {																// Begin class declaration

private:
	LPD3DXMESH pMsh;														// D3DX wrapper for mesh data
	D3DMATERIAL9 *pMat;														// List of materials for each segment
	LPDIRECT3DTEXTURE9 *pTex;												// List of textures required	
	DWORD dwNumMaterials;													// How many materials+textures are used
	
	D3DXVECTOR3 vecPosition;												// Current translation for mesh
	D3DXVECTOR3 vecScale;													// Current scale for mesh
	D3DXVECTOR3 vecRotate;													// Current rotation for mesh (in degrees)
	D3DXVECTOR3 vecCentre;													// Centre of the mesh ( in model space )
	float fSphereSize;														// Radius of the bounding sphere ( in model space )

	D3DXMATRIXA16 matMesh;													// The transformation matrix used when rendering
	bool bRebuildMatrices;													// Only rebuild the matrices if somethings changed

public:
	CMesh();																// Default ctor
	CMesh( LPDIRECT3DDEVICE9 dev, char* sFilename, char* sTexPath );		// ctor wraps to a Load() call
	~CMesh();																// dtor

	void Load( LPDIRECT3DDEVICE9 dev, char* sFilename, char* sTexPath );	// loads the specified mesh and textures
	void Render( LPDIRECT3DDEVICE9 dev );									// renders the stored mesh on screen

	//Information
	int trianglesInMesh();													// Returns number of triangles making up the mesh
	int verticesInMesh();													// Returns number of vertices making up the mesh
	LPD3DXMESH getMesh();													// Returns the internal mesh struct
	D3DXVECTOR3 getBoundingSphereCentre();									// Returns the middle point ( in world space )
	float getBoundingSphereSize();											// Returns the radius of the bounding sphere ( in world space )

	//External transformation modifiers
	void TransformMatrix( D3DXMATRIX mat );									// Sets the internal transformation matrix
	D3DXMATRIX TransformMatrix();											// gets the internal transformation matrix

	//Position Modifiers
	void positionX( float x );												// get or set ONLY the X translation
	float positionX();

	void positionY( float y );												// get or set ONLY the Y translation
	float positionY();

	void positionZ( float z );												// get or set ONLY the Z translation
	float positionZ();

	void position( float x, float y, float z );								// get or set all components of the translation
	D3DXVECTOR3 position( );

	//Scale Modifiers
	void scaleX( float x );													// get or set ONLY the X scale
	float scaleX();

	void scaleY( float y );													// get or set ONLY the Y scale
	float scaleY();

	void scaleZ( float z );													// get or set ONLY the Z scale
	float scaleZ();

	void scale( float x, float y, float z );								// get or set all scalars at the same time
	D3DXVECTOR3 scale( );

	//Rotation Modifiers
	void rotationX( float x );												// get or set X rotation (in degrees)
	float rotationX();

	void rotationY( float y );												// get or set Y rotation (in degrees)
	float rotationY();

	void rotationZ( float z );												// get or set Z rotation (in degrees)
	float rotationZ();

	void rotation( float x, float y, float z );								// get or set all rotations (in degrees)
	D3DXVECTOR3 rotation( );

};																			// finish class definition


#endif //INC_CMESH_H