/*
 *
 *	Core.cpp
 *	Started: 1st August 2003
 *	Coded by: Jack Hoxley
 *	
 */

//a necessary include when writing Win32 programs
#include <Windows.h>
#include <exception>
#include <stdio.h>
#include "CGraphics.h"
#include "CGraphicsException.h"

#define WINDOWED_MODE	//comment this out to get fullscreen.

//global variables. ugly as hell, but I cant
//be bothered to sort them out right now!
CGraphics *pGFXEngine;

LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch( msg )
    {
		//we aren't interested in many messages for this sample app.
        case WM_DESTROY:
            PostQuitMessage( 0 );
            return 0;
			break;

		case WM_KEYDOWN:
			//a key was depressed
			//wParam:	virtual key-code pressed (VK_***)
			switch ( wParam ) {
				case VK_ESCAPE:
					//exit program
					PostQuitMessage( 0 );
					break;

				case VK_LEFT:
					pGFXEngine->ModelDetail( pGFXEngine->ModelDetail() - 1 );
					break;
				case VK_RIGHT:
					pGFXEngine->ModelDetail( pGFXEngine->ModelDetail() + 1 );
					break;
				case 'W':
					//wireframe
					pGFXEngine->Wireframe(!pGFXEngine->Wireframe());
					break;
				case 'P':
					//animate camera
					pGFXEngine->AnimateCamera(!pGFXEngine->AnimateCamera());
					break;
				case 'T':
					//text level
					pGFXEngine->TextOutput( pGFXEngine->TextOutput() + 1 );
					break;
				case 'S':
					//statistics
					pGFXEngine->StatisticsOutput( !pGFXEngine->StatisticsOutput() );
					break;

				case '1':
					//ambient lighting enabled?
					pGFXEngine->Light( 0, !pGFXEngine->Light( 0 ) );
					break;

				case '2':
					pGFXEngine->Light( 1, !pGFXEngine->Light( 1 ) );
					break;

				case '3':
					pGFXEngine->Light( 2, !pGFXEngine->Light( 2 ) );
					break;

				case '4':
					pGFXEngine->Light( 3, !pGFXEngine->Light( 3 ) );
					break;

				case '5':
					pGFXEngine->Light( 4, !pGFXEngine->Light( 4 ) );

				default:
					break;
			}
			return 0;
			break;
    }

    return DefWindowProc( hWnd, msg, wParam, lParam );
}


//WinMain - the entry point for this application. execution starts here
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT ) {

	//0. Any variables;
	DWORD wStyle = WS_VISIBLE | WS_CAPTION | WS_MINIMIZEBOX | WS_SYSMENU;
	WNDCLASSEX window;								//holds everything we know about our window TEMPLATE.
	HWND hWnd;										//holds a reference to the CREATED instance of the window
	
	//1. Configure the window class
	ZeroMemory( &window, sizeof(WNDCLASSEX) );		//set all unneeded members to NULL/0
	window.cbSize = sizeof(WNDCLASSEX);				//store the size of the variable
	window.lpszClassName = "Direct3D9Cpp";			//give it a unique name. the user wont see this.
	window.hInstance = GetModuleHandle(NULL);		//get a default module handle.
	window.lpfnWndProc = MsgProc;					//IMPORTANT: tell Win32 where to send messages.
	window.style = CS_HREDRAW | CS_VREDRAW;
	window.hbrBackground = (HBRUSH)GetStockObject( BLACK_BRUSH );
	window.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	window.hIconSm = LoadIcon(NULL, IDI_WINLOGO);
	window.hCursor = LoadCursor(NULL, IDC_ARROW);
	
	//2. register it with the system
	RegisterClassEx( &window );						//register the class with the system.

	//3. create the window for use
	DWORD winW   = 600 + (GetSystemMetrics( SM_CXFIXEDFRAME ) * 2);    
	DWORD winH	 = 450 + (GetSystemMetrics( SM_CYFIXEDFRAME ) * 2) + GetSystemMetrics( SM_CYCAPTION );
	hWnd = CreateWindow("Direct3D9Cpp",				//refer back to the name we gave the class earlier
						"D3D9 Stencil Shadows",		//this is the window caption - the user sees this.
						wStyle,						//style of window to use.
						0, 0,						// TOP LEFT COORDINATE
						winW, winH,					// BOTTOM RIGHT COORDINATE
						GetDesktopWindow(),			//parent window will be the windows desktop
						NULL,						//we aren't using menu's so we pass NULL
						window.hInstance,			//handle to the original class
						NULL);						//not important. for MDI windows.

	//4. show the window on screen
	ShowWindow(hWnd,SW_SHOWDEFAULT);				//display the window on screen.
	UpdateWindow( hWnd );							//update it / send it initial msgs.

	//5. create the graphics engine
	try {
#ifdef WINDOWED_MODE
		pGFXEngine = new CGraphics(hWnd);
#else
		if(CGraphics::IsDisplayModeOkay(1024,768,32))
			pGFXEngine = new CGraphics(hWnd, 1024,768,32);
		else
			if(CGraphics::IsDisplayModeOkay(1024,768,16))
				pGFXEngine = new CGraphics(hWnd, 1024,768,16);
			else
				if(CGraphics::IsDisplayModeOkay(640,480,32))
					pGFXEngine = new CGraphics(hWnd, 640,480,32);
				else
					if(CGraphics::IsDisplayModeOkay(640,480,16))
						pGFXEngine = new CGraphics(hWnd, 640,480,16);
					else
						//fail!!!
						throw CGraphicsException("Could not find a compatable fullscreen display mode","Core.cpp",__LINE__);
#endif

	} catch ( CGraphicsException x ) {
		//catch a specific engine-specified error message
		x.ReportError();
		//clean up and exit:
		SAFE_DELETE( pGFXEngine );
		UnregisterClass( "Direct3D9Cpp", window.hInstance );
		return 0;
	} catch ( exception err ) {
		//catch a generic erorr message
		MessageBox(hWnd,err.what(),"Error",MB_OK);
		MessageBox(hWnd,"Unknown exception was caught; this application will now exit.","Error",MB_OK);
		//clean up and exit:
		SAFE_DELETE( pGFXEngine );
		UnregisterClass( "Direct3D9Cpp", window.hInstance );
		return 0;
	}

	pGFXEngine->Wireframe(false);
	pGFXEngine->Light( 0, true );
	pGFXEngine->Light( 1, true );
	pGFXEngine->Light( 2, true );
	pGFXEngine->Light( 3, true );
	pGFXEngine->Light( 4, true );

	//6. enter the main application loop
	MSG winMsg;														//create a msg structure that we can use
	ZeroMemory( &winMsg, sizeof(winMsg) );							//clear its memory
	while (winMsg.message != WM_QUIT) {								//loop until we're told to quit
		if (PeekMessage( &winMsg, NULL, 0U, 0U, PM_REMOVE )) {		//check for any new messages
			TranslateMessage( &winMsg );							//process them.
			DispatchMessage( &winMsg );
		} else {													//no windows messages. lets get on with D3D graphics
			pGFXEngine->OneFrameUpdate();							//update maths/physics etc... for the current frame.
			pGFXEngine->OneFrameRender();							//render the current frame to the screen.
		}
	}

	//7. tidy up
	SAFE_DELETE( pGFXEngine );
	UnregisterClass( "Direct3D9Cpp", window.hInstance );			//tell the system we're no longer using the window
	return 0;														//exit the program.
}